Imports System.Windows.Forms
Imports System.IO

Public Class ListViewEx

    Inherits ListView
    'Dim KeyCollection As Hashtable
    Dim KeyArrayList As ArrayList

    ''' <summary>
    ''' Legt die Keys der selektierten Eintrge in einer Collection ab (Um die Selektion wieder herzustellen, rufen Sie nach Auruf dieser Sub die Sub "RestoreSelectedKeys" auf)
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub SaveSelectedKeys()
        'KeyCollection = New Hashtable
        KeyArrayList = New ArrayList

        'alle alten Selektionen entfernen
        For Each selItems As System.Windows.Forms.ListViewItem In Me.SelectedItems
            'KeyCollection.Add(selItems.Name, selItems.Name)
            KeyArrayList.Add(selItems.Name)
        Next selItems

    End Sub
    ''' <summary>
    ''' Stellt die selektion von Listview.Items anhand deren Key (mssen eindeutig sein) wieder her (Zurvor mssen Sie die Sub "SaveSelectedKeys" aufrufen, um die Selektion zu speichern)
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub RestoreSelectedKeys()
        Dim returnItem As ListViewItem()
        Dim itemX As System.Windows.Forms.ListViewItem = Nothing
        Dim Key As String

        'alle alten Selektionen entfernen
        For Each selItems As System.Windows.Forms.ListViewItem In Me.SelectedItems
            selItems.Selected = False
        Next selItems

        If KeyArrayList.Count > 0 Then
            For Each Key In KeyArrayList
                '*** Find Key and delete this with the index-number
                returnItem = Me.Items.Find(Key, False)
                If returnItem.Length = 0 Then
                    KeyArrayList.Remove(Key)
                Else
                    itemX = Me.Items(returnItem(0).Index)
                    'Eintrag markieren
                    itemX.Selected = True
                End If
            Next
        End If

    End Sub

    ''' <summary>
    ''' Diese Funktion liefert True zurck, wenn der bergeben Key in der Listview gefunden wird
    ''' </summary>
    ''' <param name="Key">Name des Keys</param>
    ''' <param name="searchAllSubItems">Bestimmt ob alle Subitems in die Suche eingeschlossen werden soll</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ContainsKey(ByVal Key As String, Optional ByVal searchAllSubItems As Boolean = False) As Boolean
        Dim returnItem As ListViewItem()

        '*** Find Key and delete this with the index-number
        returnItem = Me.Items.Find(Key, searchAllSubItems)
        If returnItem.Length = 0 Then Return False Else Return True

    End Function

    ''' <summary>
    ''' Sucht nach dem bergebenen Key und selektiert den Eintrag wenn dieser gefunden wird
    ''' </summary>
    ''' <param name="Key">zu suchender Key aus Listview</param>
    ''' <param name="searchAllSubItems">bestimmt ob SubItems in die Suche nach dem Key einbezogen werden sollen</param>
    ''' <param name="ClearExistingSelection">Wenn True, wird eine bestehende Selektion vorm selektieren des Keys entfernt</param> 
    ''' <param name="EnsureVisible">Stellt sicher, dass dass Element im Steuerelement angezeigt wird und fhrt ggf. einen Bildlauf durch durch den Inhalt des Steuerelements durch</param>
    ''' <remarks></remarks>
    Public Sub SelectByKey(ByVal Key As String, Optional ByVal searchAllSubItems As Boolean = False, Optional ByVal ClearExistingSelection As Boolean = False, Optional ByVal EnsureVisible As Boolean = False)
        Dim returnItem As ListViewItem()
        Dim itemX As System.Windows.Forms.ListViewItem = Nothing


        '*** Find Key and delete this with the index-number
        returnItem = Me.Items.Find(Key, searchAllSubItems)
        If returnItem.Length = 0 Then Exit Sub Else itemX = Me.Items(returnItem(0).Index)

        'alle alten Selektionen entfernen (optional)
        If ClearExistingSelection = True Then
            For Each selItems As System.Windows.Forms.ListViewItem In Me.SelectedItems
                selItems.Selected = False
            Next selItems
        End If

        'Eintrag markieren
        itemX.Selected = True
        If EnsureVisible = True Then itemX.EnsureVisible()

    End Sub

    ''' <summary>
    ''' Setzt die Schriftfarbe fr eine ganze Zeile
    ''' </summary>
    ''' <param name="lRow">Zeilennummer fr die die Schriftfarbe gesetzt werden soll</param>
    ''' <param name="ForeColor">gewnschte Farbe</param>
    ''' <param name="bRefresh">Wenn True, erfolgt ein neu Zeichnen der ListView. Bei mehreren nderungen sollten Sie erst alle durchfhren und nur am Ende ein neuzeichnen durchfhren.</param>
    ''' <remarks></remarks>
    Public Sub SetRowForeColor(ByVal lRow As Integer, ByVal ForeColor As System.Drawing.Color, Optional ByRef bRefresh As Boolean = False)

        Dim itemX As System.Windows.Forms.ListViewItem = Me.Items.Item(lRow)

        itemX.UseItemStyleForSubItems = True 'Wichtig damit alle SubItems die Formatierung Items bernimmt.

        If itemX.ForeColor <> ForeColor Then
            itemX.ForeColor = ForeColor
            If bRefresh = True Then Me.Refresh() ' Wichtig! Neuzeichnen
        End If

    End Sub
    ''' <summary>
    ''' Setzt die Hintergrundfarbe fr eine ganze Zeile
    ''' </summary>
    ''' <param name="lRow">Zeilennummer fr die die Hintergrundfarbe gesetzt werden soll</param>
    ''' <param name="BackColor">gewnschte Farbe</param>
    ''' <param name="bRefresh">Wenn True, erfolgt ein neu Zeichnen der ListView. Bei mehreren nderungen sollten Sie erst alle durchfhren und nur am Ende ein neuzeichnen durchfhren.</param>
    ''' <remarks></remarks>
    Public Sub SetRowBackColor(ByVal lRow As Integer, ByVal BackColor As System.Drawing.Color, Optional ByRef bRefresh As Boolean = False)
        ' Hintergrundfarbe einer Zeile setzen

        Dim itemX As System.Windows.Forms.ListViewItem = Me.Items.Item(lRow)

        itemX.UseItemStyleForSubItems = True 'Wichtig damit alle SubItems die Formatierung Items bernimmt.

        If itemX.BackColor <> BackColor Then
            itemX.BackColor = BackColor
            If bRefresh = True Then Me.Refresh() ' Wichtig! Neuzeichnen
        End If

    End Sub

    ''' <summary>
    ''' Speichert den Inhalt der ListView in eine Textdatei
    ''' </summary>
    ''' <param name="sFile">Zieldatei inkl. Pfad</param>
    ''' <param name="sDelimiter">Trennzeichen zwichen den Spalten z.B. ;</param>
    ''' <param name="bDeleteExistingFile">Wenn Wert True ist, wird eine vorhandene Datei (mit gleichem Namen) ohne Rckfrage gelscht</param>
    ''' <param name="sHeaderText">Optional: Ein Text der ber der Tabelle ausgegeben werden soll</param>
    ''' <param name="sFooterText">Optional: Ein Text der unter der Tabelle ausgegeben werden soll</param>
    ''' <param name="bWriteTimeStampEveryRow">Wenn Wert True ist, wird in jede Zeile noch eine Spalte mit einem Timestamp aufgenommen.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function SaveToFile(ByVal sFile As String, Optional ByVal sDelimiter As String = Nothing, Optional ByVal bDeleteExistingFile As Boolean = True, Optional ByVal sHeaderText As String = "", Optional ByVal sFooterText As String = "", Optional ByVal bWriteTimeStampEveryRow As Boolean = False) As Boolean
        'Ist die Datei vorhanden, werden die Eintrge angehangen
        If sDelimiter = Nothing Then sDelimiter = ControlChars.Tab

        Dim intCols As Short
        Dim sPath As String = sFile.Substring(0, sFile.Length - (sFile.Length - sFile.LastIndexOf("\")) + 1)
        Dim i As Short, U As Short
        Dim bFileExist As Boolean = False
        Dim sTimestamp As String = System.DateTime.Today.Year.ToString("D4") & "-" & System.DateTime.Today.Month.ToString("D2") & "-" & System.DateTime.Today.Day.ToString("D2") & " " & System.DateTime.Now.Hour.ToString("D2") & ":" & System.DateTime.Now.Minute.ToString("D2") & ":" & System.DateTime.Now.Second.ToString("D2") & "," & System.DateTime.Now.Millisecond.ToString("D2")

        'Ordner (Pfad) erstellen, falls dieser nicht vorhanden ist
        If Directory.Exists(sPath) = False Then Directory.CreateDirectory(sPath)
        Dim objDateiMacher As StreamWriter
        If File.Exists(sFile) Then
            bFileExist = True
            If bDeleteExistingFile = True Then File.Delete(sFile)
        End If


        objDateiMacher = File.AppendText(sFile)

        'Optionalen Header ausgeben
        If sHeaderText.Length <> 0 Then
            objDateiMacher.Write(sHeaderText)
            objDateiMacher.Write(Environment.NewLine & Environment.NewLine)
        End If

        ' Anzahl Spalten im ListView (In diesem Fall ist die letzte Spalte eine Hilfsspalte dich nicht mit ausgegeben werden soll)
        intCols = Me.Columns.Count - 1

        ' ListView zeilenweise speichern
        With Me.Columns
            'Spaltenkpfe ausgeben (wenn Datei vorhanden war Kpfe nicht ausgebben)
            If bFileExist = False Then
                If bWriteTimeStampEveryRow = True Then objDateiMacher.Write("Timestamp" & sDelimiter)

                For U = 0 To intCols
                    If .Item(U).Width > 0 Then 'Ausgeblendete Spalten nicht ausgeben
                        If U > 0 Then objDateiMacher.Write(sDelimiter)
                        objDateiMacher.Write(.Item(U).Text)
                    End If
                Next U
            End If
            ' Zeilenvorschub
            objDateiMacher.Write(Environment.NewLine)

            For i = 0 To Me.Items.Count - 1
                With Me.Items.Item(i)
                    'Timestamp speichern (optional)
                    If bWriteTimeStampEveryRow = True Then objDateiMacher.Write(sTimestamp & sDelimiter)
                    ' dann wird der Text der 1. Spalte gespeichert
                    objDateiMacher.Write(.Text)
                    ' Jetzt alle weiteren Spalten-Inhalte
                    For U = 1 To intCols
                        If Me.Columns.Item(U).Width > 0 Then objDateiMacher.Write(sDelimiter & .SubItems(U).Text)
                    Next U
                    'Zeilenvorschub
                    If i <> Me.Items.Count - 1 Then objDateiMacher.Write(Environment.NewLine)
                End With
            Next i
        End With

        'Optionalen Footer ausgeben
        If sFooterText.Length <> 0 Then
            objDateiMacher.Write(Environment.NewLine & Environment.NewLine)
            objDateiMacher.Write(sFooterText)
        End If

        objDateiMacher.Close()
        objDateiMacher = Nothing
        Return True

    End Function
    ''' <summary>
    ''' Fhrt BeginUpdate und SuspendLayout aus
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub StopRefresh()
        With Me
            .BeginUpdate()
            .SuspendLayout()
        End With
    End Sub
    ''' <summary>
    ''' Fhrt ResumeLayout und EndUpdate aus
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ResumeRefresh()
        With Me
            .ResumeLayout()
            .EndUpdate()
        End With
    End Sub

    ''' <summary>
    ''' Lscht alle Zeilen, die den angegeben Text nicht in der angegebenen Spalten enthalten
    ''' </summary>
    ''' <param name="sText">Text der in einem Feld enthalten sein muss</param>
    ''' <param name="iColumnIndex">Spalte auf die die Funktion angewendet werden soll</param>
    ''' <param name="bIgnoreCase">Gibt an ob Gross-/Kleinschreibung ignoriert werden soll</param>
    ''' <param name="bUseContainsInsteadOfStringCompare">Bei True wird ein Contains-Prfung anstatt kompletter String vergleich durchgefhrt</param>
    ''' <remarks></remarks>
    Public Sub DeleteRowsThatDontContainsString(ByVal sText As String, ByVal iColumnIndex As Integer, Optional ByVal bIgnoreCase As Boolean = True, Optional ByVal bUseContainsInsteadOfStringCompare As Boolean = False)
        Dim oListviewItem As ListViewItem = Nothing
        Dim oListView As ListView = Me

        If bUseContainsInsteadOfStringCompare = True Then
            If bIgnoreCase = True Then
                For Each oListviewItem In Me.Items
                    If oListviewItem.SubItems(iColumnIndex).Text.ToLower.Contains(sText.ToLower) = False Then oListviewItem.Remove()
                Next
            Else
                For Each oListviewItem In Me.Items
                    If oListviewItem.SubItems(iColumnIndex).Text.Contains(sText) = False Then oListviewItem.Remove()
                Next
            End If
        Else
            For Each oListviewItem In Me.Items
                If String.Compare(oListviewItem.SubItems(iColumnIndex).Text, sText, bIgnoreCase) <> False Then oListviewItem.Remove()
            Next
        End If


    End Sub

End Class
